
# This file handles the user interface.  It is a replacement for the old
# gtbrew interface, which just got to be a PITA, and didn't cleanly lend
# itself toward any upgrades.  So, use an interface based on the brewing
# log sheets I created (based on the ones Todd uses).  It also handles
# actions related to the display.

source lib/lsprocs.tcl
set topdir [pwd]

########################################################################
###  Header info
########################################################################

btxt "Name of beer:" 15 14   ; oneval 121 14 Name 32
btxt "Date Brewed:"  350 14  ; set wtm [oneval 445 14 Date 22]
btxt Style: 15 47            ; oneval 68 47 Style 32

bind $wtm <3> { set Date $dateonly }

button .c.stq -text {[?]} -relief flat -background white -padx 0 -pady 0 \
      -foreground blue -command { if {[string length $Style]} { readstyle } }
.c create window 274 47 -window .c.stq -anchor w

set bjcp 1
radiobutton .c.bjcp -variable bjcp -value 1 -text BJCP \
   -font $bjcpbox -command { styles_active normal }
.c create window 339 47 -window .c.bjcp  -anchor w

radiobutton .c.stna -variable bjcp -value 0 -text N/A \
   -font $bjcpbox -command { styles_active disabled }
.c create window 390 47 -window .c.stna  -anchor w



########################################################################
###  Grain bill entry
########################################################################

btxt GRAINS: 15 77

# This creates the rows of data for the grain bill
proc grainrow {x y idx} {
   global Grain newgrains small_labels small_labels

   grainval $x $y Grain($idx:lb)
   incr x 29
   .c create text $x $y -text lbs -font $small_labels -anchor w
   incr x 19
   grainval $x $y Grain($idx:oz) 4
   incr x 33
   .c create text $x $y -text oz -font $small_labels -anchor w
   incr x 28

#  Create our ComboBox for the grains here
   ComboBox .c.e$x:$y -values $newgrains() -width 16 -listboxwidth 300 -height 20 -hottrack false \
   -textvariable Grain($idx:name) -font $small_labels \
   -highlightcolor white -highlightbackground black \
   -selectbackground white -selectforeground black \
   -entrybg white -editable true -bwlistbox false
   .c create window $x $y -window .c.e$x:$y -anchor w

   incr x 110
   label .c.pc$x:$y -textvariable Grain($idx:percent) -background white \
      -foreground black -font $small_labels
   .c create window $x $y -window .c.pc$x:$y -anchor w

   incr x 28
   .c create text $x $y -text % -font $small_labels -anchor w
}

# preserve this order to keep <tab> moving the cursor the right way
grainrow 81  74 L1
grainrow 81  97 L2
grainrow 81 124 L3
grainrow 81 150 L4
grainrow 81 181 L5
grainrow 344  74 R1
grainrow 344  97 R2
grainrow 344 124 R3
grainrow 344 150 R4
grainrow 344 181 R5

button .c.calc -text "\[Calc]" -relief $relief -background white -padx 0 \
   -foreground black -font $small_labels -command {calc_grain}
.c create window 15 113 -window .c.calc -anchor w

set Grain() [list L1 L2 L3 L4 L5 R1 R2 R3 R4 R5]


########################################################################
###  Hop schedule entry
########################################################################

btxt HOPS 8 259

# Add an indicator on the hops entry stating that it is expecting OUNCES
.c create text 14 276 -text "(oz)" -anchor w -font $small_labels \
   -fill blue

# Draw the grid for the Hop Additions (I LIKE my table!)

.c create rectangle 52 212 589 472
.c create line 52 212 589 212 -width 2
foreach y [list 240 269 298 327 356 385 414 443 472] {
   .c create line 52 $y 589 $y
}
foreach x [list 118 399 444 497 540] {
   .c create line $x 212 $x 472
}
btxt Amount 57 231
btxt "Hop Variety" 202 231
btxt "%AA"   405 231
btxt Time    452 231
btxt IBU     502 231
btxt "%IBU"  546 231

button .c.hcalc -text "\[Calc]" -relief $relief -background white \
   -foreground black -font $small_labels -command {calc_hops} \
   -padx 0
.c create window 5 310 -window .c.hcalc -anchor w

set Hop() [list H1 H2 H3 H4 H5 H6 H7 H8]

# preserve this order to keep <tab> moving the cursor the right way
set y 254
set i 1
foreach idx $Hop() {
   set amtw($idx) [onehop 62 $y Hop($idx:amount) 5 ]
   $amtw($idx) configure -state normal -disabledforeground black \
         -disabledbackground white
   ComboBox .c.hv$i -values $hops() -width 32 -height 20 -hottrack false \
      -textvariable Hop($idx:variety) -font $small_text \
      -highlightcolor white -highlightbackground black \
      -selectbackground white -selectforeground black \
      -entrybg white -modifycmd "sethop $idx" -editable true \
      -autopost true -bwlistbox false

   .c create window 128 $y -window .c.hv$i -anchor w
   onehop 405 $y Hop($idx:aa) 3
   onehop 458 $y Hop($idx:time) 3
   balloonhelp .c.e458:$y \
"Enter:
   a) time in minutes until end of boil (EOB),
   b) \"fwh\" for first wort hops,
   c) or \"eob\" for end of boil addition(s)"

   set ibuw($idx) [onehop 505 $y Hop($idx:ibu) 3]
   $ibuw($idx) configure -state disabled -disabledbackground white \
      -disabledforeground black -border 0

   incr i
   incr y 29
}

# This is done apart from the others...again, for <tab> order
set y 254
foreach idx $Hop() {
   onelbl 548 $y Hop($idx:percent) 3
   incr y 29
}

set gethopmode 0
radiobutton .c.getibu -text "IBUs" -variable gethopmode -value 0 \
   -font $small_text -command {
   foreach idx $Hop() {
      $amtw($idx) configure -state normal -border 1
      $ibuw($idx) configure -state disabled -border 0
      .c.hcalc configure -command {calc_hops}
   }
}
.c create window 1 350 -window .c.getibu -anchor w
radiobutton .c.getamt -text "AMT" -variable gethopmode -value 1 \
   -font $small_text -command {
   foreach idx $Hop() {
      $amtw($idx) configure -state disabled -border 0
      $ibuw($idx) configure -state normal -border 1
      .c.hcalc configure -command {calc_hops 1}
   }
}
.c create window 1 385 -window .c.getamt -anchor w

balloonhelp .c.getibu "Given the amount, get the IBUs for each hop addition"
balloonhelp .c.getamt "Given the IBUs, get the amount (oz) for each hop addition"

########################################################################
###  Yeast and Info Lines (dilution, stepmash, etc.) entry
########################################################################

### Yeast

btxt YEAST: 14 507
oneval 75 507 yeast 20
# Button for getting Yeast info when I get that far
button .c.ystq -text {[?]} -relief flat -command { get_yeast_info }
   .c.ystq configure -background white -foreground blue -padx 0 -pady 0
.c create window 205 506 -window .c.ystq -anchor w

### Batch size

btxt "Batch size (gal):" 265 507
label .c.bstxt -textvariable batchsize -foreground #000000 \
   -background white -font $labels
.c create window 395 507 -window .c.bstxt -anchor w

### Info

set stepmash "Strike (single-infusion):"
label .c.stepmash -textvariable stepmash -font $labels \
   -background white -foreground #cc0000 -anchor w -justify left
.c create window 5   535 -window .c.stepmash -anchor w
.c.stepmash configure -state disabled

label .c.dilution -textvariable diluvals -width 75 -font $labels \
   -anchor w -justify left
.c.dilution configure -background white -foreground #cc0000 -state disabled
.c create window 5  560 -window .c.dilution -anchor w


########################################################################
###  TARGET SPECIFICATIONS
########################################################################

btxt "TARGET SPECIFICATIONS:"  608 23

### OG

btxt "Original Gravity:"  629 55
txt Target:  786 56                  ; onearlbl 848 55  ogtarget ogtarget
                                       onearlbl 898 55  ogtargpl ogtargpl
txt Range:   786 79                  ; onearlbl 848 75  ogrange ogrange

### FG

btxt "Final Gravity:" 629 102
txt Min:  786 102                    ; onearlbl 825 100 fgmin fgmin
txt Max:  914 102                    ; onearlbl 952 100 fgmax fgmax

### SRM

btxt "Color (SRM):"  629 129
txt Target:  786 129
txt (Calculated)  925 129            ; onearlbl 848 128 srmtarget srmtarget
txt Range:   786 161                 ; onearlbl 836 159 srmrange  srmrange
label .c.srmimg -text "          " -width 20 -font $tiny_text
.c create window 628 154 -window .c.srmimg -anchor w
.c create text 650 174 -text "(Approx. Color)" -font $small_text \
   -anchor w -fill blue

### IBUs

btxt "Bitterness (IBUs):"  629  192
txt Target:  786 192                 ;# onearlbl 848 193 ibutarget ibutarget
label .c.ibutotal -textvariable display_ibus -font $labels \
         -background white -foreground black -width 5
   .c create window 852 189 -window .c.ibutotal -anchor w
txt (Calculated)  925 192
txt Range:  786 221                  ; onearlbl 848 219 iburange  iburange



########################################################################
###  BREW DAY LOGS / ACTUAL SPECS
########################################################################

### Mash

btxt Mash    612 260
txt Strike:  655 260     ; oneval 711 260 striketemp 5
txt  F      752 260
txt Volume:  777 260     ; oneval 835 260 strikevol 5
txt gal. 885 260
txt "Mash temp:" 655 279 ; oneval 732 279 actualmashtemp 5
txt  F      774 279

### Sparge

btxt SPARGE:  612 304
txt Temp:  680 304            ; oneval 723 304 spargetemp 5
txt  F   764 304
txt "WORT 1:"  783 304        ; oneval 848 304 firstwort 4
txt  P 881    304
txt "LAST WORT:"  655 337     ; oneval 742 337 lastwort 4
txt  P  774 337
txt "Yield (gal at sparge temps):"  612 362   ; oneval 803 362 preboilvol 8
txt "Spec. Grav.:"  612 390   ; oneval 701 390 preboilsg 8
txt "OR as P" 763 390 ; oneval 842 390 pbsgdp 6
button .c.pbsgok -text OK -background white -foreground black \
   -command { set preboilsg [format %.3f [Plato_to_SG $pbsgdp]] }
.c create window 902 390 -window .c.pbsgok -anchor w


### Actual specs

btxt "ACTUAL SPECIFICATIONS:"  612 430
txt "Final Volume (gal at boil temps):" 638 463
oneval 850 463 finalvol 7

txt OG:  638 501 ; oneval 683 501 ogactual 10
txt FG:  781 501 ; oneval 826 501 fgactual 10
label .c.abv -textvariable abv -font $labels -relief flat -background white -foreground black
label .c.endmasheff -textvariable endmasheff  -font $labels -relief flat -background white -foreground black
label .c.mashppg -textvariable mashppg   -font $labels -relief flat -background white -foreground black
.c create window 612 533 -window .c.abv -anchor w
.c create window 612 555 -window .c.endmasheff -anchor w
.c create window 781 555 -window .c.mashppg -anchor w

set endmasheff "Mash eff.:  xx%"
set mashppg "Mash pts/lb/gal:  xx"

### Mash efficiency and pts/lb

# set rlf $relief
# set relief flat
# onelbl 612 555 endmasheff 20
# onelbl 804 555 mashppg 20
# set relief $rlf

label .c.dtm -textvariable datetime -foreground #00ffff -background black \
   -relief $relief
.c create window 1000 23 -window .c.dtm -anchor e

# foreach t [list batchsize OGSG preboilvol preboilsg] {
#    trace variable $t w { checkdilu }
# }
# 
# trace variable ogactual w { checkeff ; checkogfg }
# trace variable $fgactual w { checkogfg }
# 
# foreach t [list finalvol totalweight] {
#    trace variable $t w { checkeff }
# }


.c.bjcp invoke

