

# In order to avoid problems...we only WRITE to resdb (invdb is
# read-only).  Likewise, GT-BrewDB treats resdb as read-only.


proc load_resdb {} {
   global resdb invok Name dbhome

   if {!$invok} { return }
   if {[check_dblock]} {
set msg "GTbrew2 has determined that GT-BrewDB is currently running. \
If you make changes from either while both have the database open, one \
or both sets of changes may be lost.  Please close GT-BrewDB before \
proceeding any further, or you may trash your inventory and/or \
reserved grains/hops databases...."

      tk_messageBox -type ok -message $msg
   }
   if {[check_dblock]} {
      set s [tk_messageBox -type yesno \
         -message "Are you SURE you wish to continue?"]

      if {[string compare -nocase $s "yes"]} {
         set invok 0 ; return
      }

   }
   db resdb
   source $dbhome/reserved.db
}

proc load_invdb {} {
   global dbhome invok Name
   if {!$invok} { return }

   db invdb
   source $dbhome/inventory.db
   set ilist [lsort -dictionary [invdb]]
}


proc save_invdb {} {
   global invdb dbhome
   set f [open $dbhome/inventory.db w]

   foreach idx [invdb] {
      set class [invdb $idx class]
      if {[streq $class grain]} {
         set name [invdb $idx name]
         set amt  [invdb $idx amt]

         puts $f "set idx \"$idx\""
         puts $f "invdb \"\$idx\" name \"$name\""
         puts $f "invdb \"\$idx\" amt $amt"
         puts $f "invdb \"\$idx\" class grain"
         puts $f ""
      }
   }
   foreach idx [invdb] {
      set class [invdb $idx class]
      if {[streq $class hop]} {
         set name  [invdb $idx name]
         set amt   [invdb $idx amt]
         set alpha [invdb $idx alpha]
         set type  [invdb $idx type]

         puts $f "set idx \"$idx\""
         puts $f "invdb \"\$idx\" name \"$name\""
         puts $f "invdb \"\$idx\" amt $amt"
         puts $f "invdb \"\$idx\" alpha $alpha"
         puts $f "invdb \"\$idx\" type $type"
         puts $f "invdb \$idx class hop"
         puts $f ""
      }
   }
   foreach idx [invdb] {
      set class [invdb $idx class]
      if {[streq $class yeast]} {

         set strain [invdb $idx strain]
         set name  [invdb $idx name]
         set lab   [invdb $idx lab]
         set master [invdb $idx master]
         set working [invdb $idx working]

         puts $f "set idx \"$idx\""
         puts $f "invdb \"\$idx\" strain $strain"
         puts $f "invdb \"\$idx\" name \"$name\""
         puts $f "invdb \"\$idx\" lab \"$lab\""
         puts $f "invdb \"\$idx\" master $master"
         puts $f "invdb \"\$idx\" working $working"
         puts $f "invdb \"\$idx\" class yeast"
         puts $f ""
      }
   }
   close $f
}

proc getlb {lb oz} { return [expr {1.000 * $lb + ($oz/16.000)}] }

proc getlboz {lb} {
   set l [expr {1.000 * int($lb)}]
   set oz [expr {16.000 * ($lb - $l)}]
   return [list $l $oz]
}


proc save_resdb {} {
   global resdb dbhome

   set f [open $dbhome/reserved.db w]
   puts $f [list array set resdb [array get resdb]]
   close $f
}


proc clear_reserved { {name ""}} {
   global resdb Name

   set tname $Name
   if {[string length $name]} { set tname $name }

   set namelist [resdb]
   foreach n $namelist {
      if {[streq $tname $n]} {
         resdb $tname ""
         save_resdb
         return
      }
   }

   toplevel .getname
   set w .getname
   wm geometry $w +600+400

   label $w.l1 -text "The current brew name doesn't match any in reserved.db"
   label $w.l2 -text "Please select from the following:"
   pack $w.l1 $w.l2 -side top

   set i 1
   foreach name $namelist {
      button $w.b$i -text "$name" \
         -command "destroy .getname ; clear_reserved \"$name\" "
     pack $w.b$i -side top 
      incr i
   }
   button $w.b0 -text "None of the above\n\n" -command { destroy .getname }
   pack $w.b0 -side top

}



proc use_reserve {savemode} {
   global resdb invdb ilist dbhome invok Grain Hop Name
   if {!$invok} { return }

  load_resdb
   if {!$invok} { return }

   if {$savemode == 1} {
      clear_reserved

      ;# use invdb here to subtract all grains/hops from invdb
      ;# create indexes that don't exist, allow negative values

      foreach idx $Grain() {
         set grname $Grain($idx:name)
         set lb $Grain($idx:lb) ; if {[string length $lb] == 0} { set lb 0 }
         set oz $Grain($idx:oz) ; if {[string length $oz] == 0} { set oz 0 }
         if {$lb == 0 && $oz == 0} { continue }
         set amt [expr {0.000 - [getlb $lb $oz]}]

         set glist [invdb $grname]
         if {[llength $glist] == 0} {
            invdb $grname name $grname
            invdb $grname amt $amt
            invdb $grname class grain
         } else {
            set eamt [invdb $grname amt]
            set newamt [expr {1.0000 * ($eamt + $amt)}] ;# amt is negative
            invdb $grname amt $newamt
         }
      }


      # Here, we don't need to worry about hops that are used more
      # than once, as we just cycle through and subtract the amounts
      # from invdb as we go.  Hops used more than once will just have
      # the individual amounts subtracted one by one.

      foreach idx $Hop() {
         load_invdb
         set hname $Hop($idx:variety)
         set alpha $Hop($idx:aa)
         set time  $Hop($idx:time)
         set amt   $Hop($idx:amount)  ;# amount in ounces, so we convert
         if {[string length $amt] == 0 || $amt == 0} { continue }
         set amt   [getlb 0 $amt]    ;# it here to match invdb

      # put the idx in the format used in invdb
         set regstr {(.*) \(([0-9.]*)%\) ([pl].*)$}
         set hlist [invdb $hname]
         if {[llength $hlist]} {
            set idx $hname
            set eamt [invdb $idx amt]
            set newamt [expr {1.000 * ($eamt - $amt)}]
            invdb $idx amt $newamt
         } else {
            set s1 "Is \"$hname\" $alpha % alpha at "
            set s2 "$time minutes a leaf or pellet hop?"
            set type [tk_dialog .dialog "Leaf or Pellet?" "$s1$s2" \
                        "" leaf leaf pellet]
            if {$type == 0} { set type "leaf" } else { set type pellet }
            set idx "$hname ($alpha%) $type"
            # check to see if we've just added this hop
            set tlist [invdb $idx]
            if {[llength $tlist]} {
               set eamt [invdb $idx amt]
               set newamt [expr {1.000 * ($eamt - $amt)}]
               invdb $idx amt $newamt

            } else {
               invdb $idx name $hname
               invdb $idx alpha $alpha
               invdb $idx type $type
               invdb $idx amt [expr {0.000 - $amt}]
               invdb $idx class hop
            }
         }
      save_invdb
      }
      return
   } elseif {$savemode == 3} {  clear_reserved ; return
   }

   resdb $Name brew $Name
   resdb $Name date [clock format [clock seconds] -format "%d%h%Y"]

   set graindb() [list]
   foreach idx $Grain() {
      set grname $Grain($idx:name)
      set lb $Grain($idx:lb) ; if {[string length $lb] == 0} { set lb 0 }
      set oz $Grain($idx:oz) ; if {[string length $oz] == 0} { set oz 0 }
      if {$lb == 0 && $oz == 0} { continue }

      set amt [getlb $lb $oz]
#
# Handle cases where the same grain is entered twice rather than
# simply updating its existing entry (why? no idea, but allow for
# it anyways, just in case).
#
      if {$amt > 0} {
         if {[lsearch -exact $graindb() $grname] >= 0} {
            set oldamt $graindb($grname)
            set newamt [expr {1.0000*($oldamt + $amt)}]
            set graindb($grname) $newamt)
         } else {
            lappend graindb() $grname
            set graindb($grname) $amt
         }
      }
   }
   set grainlist [list]
   foreach malt $graindb() {
      set amt $graindb($malt)
      lappend grainlist [list $malt $amt]
   }
   resdb "$Name" grains $grainlist

# Hops are often used more than once in a hop schedule, so here, we
# MUST allow for multiple entries of the same hop variety.

   set hopsdb() [list]
   set alpha() [list]
   foreach idx  $Hop() {
      set hname $Hop($idx:variety)
      set resalpha $Hop($idx:aa)
puts "Just set resalpha for $Hop($idx:variety) to $resalpha"

      set time  $Hop($idx:time)
      set amt   $Hop($idx:amount)  ;# amount in ounces, so we convert
      if {[string length $amt] == 0 || $amt == 0} { continue }
      set amt   [getlb 0 $amt]    ;# it here to match invdb


      # put the idx in the format used in invdb

      if {[regexp {(.*) \(([0-9.]*)%\) ([pl].*)$} $hname a name ralpha type]} {
         set idx $hname
puts "Just tested regexp and got hname $hname"
      } else {
         set type [tk_dialog .dialog "Leaf or Pellet?" \
         "Is \"$hname\" $resalpha % alpha at $time minutes a leaf or pellet hop?" "" leaf leaf pellet]
         if {$type == 0} { set type "leaf" } else { set type pellet }

         set idx "$hname ($resalpha%) $type"
puts "Failed regexp, set idx to $idx"
      }

      set alpha($idx) $resalpha

      if {[lsearch -exact $hopsdb() $idx] >= 0} {
         set oldamt $hopsdb($idx)
         set amt [expr {1.0000*($oldamt + $amt)}]
         set hopsdb($idx) $amt
      } else {
         lappend hopsdb() $idx
         set hopsdb($idx) $amt
      }
   }

   set hoplist [list]
   foreach hop $hopsdb() {
      set amt $hopsdb($hop)
      set resalpha $alpha($hop)
      lappend hoplist [list $hop $amt $resalpha]
puts "saving $hoplist to resdb"
   }
   resdb "$Name" hops $hoplist
   save_resdb
}


proc savenormal {f varname} {
   upvar $varname variable
   if {[string length $variable]} {
      puts $f "set $varname \"$variable\""
   }
}


# Check for text in comments other than whitespace
proc check_comments {} {
   global brewcomments ; return [cmt_test $brewcomments]
}
proc check_mfcomments {} {
   set test [.comments.f.t get 0.0 end]
   return [cmt_test $test]
}
proc cmt_test {s} {
   regsub -all {[\t\n\r ]} $s {} test
   if {[string length $test]} { return 1 }
   return 0

}


proc savebrew {} {
   global brewchanged
   global origfn          ;# filename of file we loaded, if any
   global Name            ;# Name of brew
   global Date            ;# Date brewed
   global bjcp            ;# 1 or 0 --- is this a BJCP style brew?
   global Style           ;# Style name (BJCP or otherwise)
   global Grain           ;# grain bill data:  Grain(), Grain($idx:$val)
                          ;#    where idx is L1..L5 R1..R5; val is name lb oz
   global Hop             ;# hop sched:  Hop() H1..H5; amount variety aa time
   global yeast           ;# yeast
   global savestyledata   ;# string to save for yeast style data
                          ;#
                          ;# BREW DAY LOG DATA
                          ;#
   global striketemp      ;# strike liquor temp (deg. F)
   global strikevol       ;# strike liquor volume (gallons)
   global actualmashtemp  ;# mash temperature (measured, deg. F)
   global spargetemp      ;# sparge liquor temperature
   global firstwort       ;# first wort (info only so far)
   global lastwort        ;# last wort (info only so far)
   global preboilvol      ;# pre-boil volume (gallons)
   global preboilsg       ;# pre-boil gravity
   global ogactual        ;# post-boil specific gravity
   global fgactual        ;# final gravity (post-fermentation)
   global finalvol        ;# post-boil volume
                          ;#
                          ;# USER CONFIGURATION DATA
                          ;#
   global batchsize       ;# intended batch size (gallons)
   global boilvol         ;# boil volume (gallons) for hop calculations
   global hopmethod       ;# Rager/Tinseth
   global masheff         ;# estimated mash efficiency (e.g., 75%)
                          ;#
                          ;# DATA FOR CALCULATING STRIKE TEMP/VOLUME FOR
                          ;# A SINGLE-INFUSION MASH (explanations in
                          ;# config.tcl and lib/stepmash.tcl).
                          ;# Multi-infusion mash handled from menus.
                          ;#
   global sh_grain        ;# specific heat of grain
   global ti_tun          ;# thermal inertia of mash tun
   global boil_temp       ;# boiling point of water ; by default, 212 deg. F
   global ambient_temp    ;# ambient temperature (i.e., GRAIN temperature)
   global mash_thick      ;# desired mash thickness (qts/lb)
   global mashtemp        ;# desired mash temperature
   global brewcomments    ;# comments/notes

   if {$bjcp} {
      set topdir recipes
      set dir $Style
      regsub -all {/-} $dir { shilling} foo
      regsub -all {} $foo {a} dir
      regsub -all {} $dir {o} foo
      regsub -all {/} $foo {-} dir
      regsub -all {} $dir {e} foo
      regsub -all {[, ]} $foo {_} dir
      set savedir $topdir/$dir
   } else {
      set savedir recipes/00-Other
      set savestyledata ""
   }
   if {![file exists $savedir]} { file mkdir $savedir }
   if {[string length $Date]} {
      set date [clock format [clock scan $Date] -format %Y-%h-%d]
   } else {
      set date brew_pending
   }
   set fn $date-$Name

   set f [open $savedir/$fn w]

   savenormal $f Name
   savenormal $f Date
   savenormal $f bjcp
   savenormal $f Style
   puts $f $savestyledata

   foreach idx $Grain() {
      foreach tag [list name lb oz] {
         if {[strlen $Grain($idx:$tag)] != 0} {
            savenormal $f Grain($idx:$tag)
         }
      }
   }
   foreach idx $Hop() {
      foreach tag [list variety amount aa time] {
         if {[strlen $Hop($idx:$tag)] != 0} {
            savenormal $f Hop($idx:$tag)
         }
      }
   }
   savenormal $f yeast
   savenormal $f striketemp
   savenormal $f strikevol
   savenormal $f actualmashtemp
   savenormal $f spargetemp
   savenormal $f firstwort
   savenormal $f lastwort
   savenormal $f preboilvol
   savenormal $f preboilsg
   savenormal $f ogactual
   savenormal $f fgactual
   savenormal $f finalvol
   savenormal $f batchsize
   savenormal $f boilvol
   savenormal $f hopmethod
   savenormal $f masheff
   savenormal $f sh_grain
   savenormal $f ti_tun
   savenormal $f boil_temp
   savenormal $f ambient_temp
   savenormal $f mash_thick
   savenormal $f mashtemp
   if {[check_comments]} {
      puts $f "set brewcomments \{$brewcomments\}"
   }
   close $f

   tk_messageBox -message "Saved to $savedir/$fn" -type ok
   if {[regexp {brew_pending} $origfn] && [string length $Date]} {
      set rm [tk_messageBox -type yesno -default yes \
              -message "Delete original:  [file tail $origfn] ?"]
      if {[streq $rm "yes"]} { file delete $origfn }
   }
   set brewchanged 0
   wm title . "GTbrew2 v1.2 (10 January 2011) ($Name)"
}


proc clearbrew {} {
   global brewchanged
   global Name Date bjcp Style Grain Hop yeast savestyledata striketemp
   global strikevol actualmashtemp spargetemp firstwort lastwort
   global preboilvol preboilsg ogactual fgactual finalvol batchsize
   global boilvol hopmethod masheff sh_grain ti_tun boil_temp
   global ambient_temp mash_thick mashtemp OGPL OGSG SRM dSRM
   global maxppg actualppg grainbill hopsched totalibus totalpts
   global stylename ogmax ogmin fgmax fgmin abvmin abvmax srmmax srmmin
   global ibutarget ibumax ibumin iburange abvrange ogrange fgrange
   global ogtarget ogrange ogtargpl srmchart srmtarget srmrange
   global display_ibus stepmash endmasheff mashppg abv diluvals
   global brewcomments

   set ans ""
   if {$brewchanged} {
      set ans [tk_messageBox -type yesno -message \
         "Brew has been CHANGED!  Save first?"]
      if {[string compare $ans "yes"] == 0} { savebrew }
   }

   set brewcomments ""
   .mf.comments configure -foreground black
   .c.srmimg configure -background #dddddd
   set diluvals ""
   set abv "   %v/v"
   set endmasheff "Mash eff.:  xx%"
   set mashppg "Mash pts/lb/gal:  xx"
   set OGPL 0 ; set OGSG 1 ; set SRM 0 ; set dSRM 0
   set maxppg 0
   set actualppg 0
   set grainbill [list]
   set hopsched [list]
   set totalibus 0
   foreach idx [list L1 L2 L3 L4 L5 R1 R2 R3 R4 R5] { set totalpts($idx) 0 }
   set stylename "" ;# beer style name
   set ogmax 0 ; set ogmin 0 ; set fgmax 0 ; set fgmin 0 ; set abvmin 0 ; set abvmax 0
   set srmmax 0 ; set srmmin 0 ; set ibumax 0 ; set ibumin 0 ; set abvrange 0-0
   set ogrange 0-0 ; set fgrange 0-0 ; set srmrange 0-0 ; set iburange 0-0
   set ogtarget 0 ; set ogtargpl 0
   set Name "" ; set Date "" ; set bjcp "" ; set Style "" ; set yeast ""
   set savestyledata "" ; set striketemp "" ; set strikevol ""
   set actualmashtemp "" ; set spargetemp "" ; set firstwort ""
   set lastwort "" ; set preboilvol "" ; set preboilsg "" ; set ogactual ""
   set fgactual "" ; set finalvol "" ; set bjcp 1 ; set srmtarget 0
   set ibutarget 0 ; set display_ibus 0
   set stepmash "Strike (single-infusion):"

   foreach idx $Grain() {
      foreach tag [list name lb oz percent] { set Grain($idx:$tag) "" }
   }
   foreach idx $Hop() {
      foreach tag [list variety amount aa time ibu percent] {
         set Hop($idx:$tag) ""
      }
   }
   set Grain() [list L1 L2 L3 L4 L5 R1 R2 R3 R4 R5]
   set Hop() [list H1 H2 H3 H4 H5 H6 H7 H8]
   wm title . "GTbrew2 v1.2 (10 January 2011)"
   set brewchanged 0
}

proc loadbrew {{loadfn ""}} {
   global brewchanged
   global Name Date bjcp Style Grain Hop yeast savestyledata striketemp
   global strikevol actualmashtemp spargetemp firstwort lastwort
   global preboilvol preboilsg ogactual fgactual finalvol batchsize
   global boilvol hopmethod masheff sh_grain ti_tun boil_temp
   global ambient_temp mash_thick mashtemp snum set origfn topdir
   global brewcomments

   set ans ""
   if {$brewchanged} {
      set ans [tk_messageBox -type yesno -message \
         "Brew has been CHANGED!  Save first?"]
      if {[string compare $ans "yes"] == 0} { savebrew }
   }


   cd $topdir

   if {[strlen $loadfn]} {
      set fn $loadfn
   } else {
      set fn [tk_getOpenFile -initialdir recipes]
   }
   if {[streq $fn ""]} { return }
   clearbrew ;# moved down to avoid clearing the existing brew too early
   set origfn $fn
   source $fn
   checkgrains
   calc_grain
   calc_hops
   if {[check_comments]} {
      .mf.comments configure -foreground #d90000
   } else {
      .mf.comments configure -foreground black
   }
   wm title . "GTbrew2 v1.2 (10 January 2011) ($Name)"
   set brewchanged 0
}


proc make_report {{mode text-plain}} {
   global topdir
   global ibutarget iburange
   global Name            ;# Name of brew
   global Date            ;# Date brewed
   global bjcp            ;# 1 or 0 --- is this a BJCP style brew?
   global Style           ;# Style name (BJCP or otherwise)
   global grains          ;# holds the grain database
   global Grain           ;# grain bill data:  Grain(), Grain($idx:$val)
                          ;#    where idx is L1..L5 R1..R5; val is name lb oz
   global Hop             ;# hop sched:  Hop() H1..H5; amount variety aa time
   global yeast           ;# yeast
   global savestyledata   ;# string to save for yeast style data
   global ogtarget ogtargpl ogrange fgmin fgmax srmtarget srmrange
                          ;#
                          ;# BREW DAY LOG DATA
                          ;#
   global striketemp      ;# strike liquor temp (deg. F)
   global strikevol       ;# strike liquor volume (gallons)
   global actualmashtemp  ;# mash temperature (measured, deg. F)
   global spargetemp      ;# sparge liquor temperature
   global firstwort       ;# first wort (info only so far)
   global lastwort        ;# last wort (info only so far)
   global preboilvol      ;# pre-boil volume (gallons)
   global preboilsg       ;# pre-boil gravity
   global pbsgdp
   global ogactual        ;# post-boil specific gravity
   global fgactual        ;# final gravity (post-fermentation)
   global finalvol        ;# post-boil volume
   global abv endmasheff mashppg
                          ;#
                          ;# USER CONFIGURATION DATA
                          ;#
   global batchsize       ;# intended batch size (gallons)
   global boilvol         ;# boil volume (gallons) for hop calculations
   global hopmethod       ;# Rager/Tinseth
   global masheff         ;# estimated mash efficiency (e.g., 75%)
                          ;#
                          ;# DATA FOR CALCULATING STRIKE TEMP/VOLUME FOR
                          ;# A SINGLE-INFUSION MASH
                          ;#
   global sh_grain        ;# specific heat of grain
   global ti_tun          ;# thermal inertia of mash tun
   global boil_temp       ;# boiling point of water ; by default, 212 deg. F
   global ambient_temp    ;# ambient temperature (i.e., GRAIN temperature)
   global mash_thick      ;# desired mash thickness (qts/lb)
   global mashtemp        ;# desired mash temperature
   global brewcomments
   global SRM
   global newmasheff

   if {[string length $Date]} {
      set date [clock format [clock scan $Date] -format %Y-%h-%d]
      set brewed 1
   } else {
      set date "Brew Pending"
      set brewed 0
   }

   cd $topdir
   set fn [tk_getSaveFile -initialdir .]
   set f [open $fn w]

   if {$mode == "text-plain"} {

      puts $f "GTbrew2 Brew Report"
      puts $f ""

      puts $f "NAME:  $Name"
      puts $f "DATE BREWED: $date"
      if {$bjcp} {
      puts $f "BJCP STYLE:  $Style"
      } else {
         if {[strlen $Style]} { puts $f "STYLE (NON-BJCP):  $Style" }
      }
      puts $f ""

      if {[check_comments]} {
         puts $f "COMMENTS:"
         puts $f ""
         set cmtlist [split $brewcomments \n]
         foreach line $cmtlist { puts $f $line }
         puts $f ""
      }

      puts $f "GRAIN BILL:"
      puts $f ""
      set maxlen 0
      foreach idx $Grain() {
         if {![grain_valid $idx]} { continue }
         set len [strlen $Grain($idx:name)]
         if {$len > $maxlen} { set maxlen $len }
      }
      foreach idx $Grain() {
         if {![grain_valid $idx]} { continue }

         set lb $Grain($idx:lb)
         if {[strlen $lb] == 0} { set lb 0 }
         set oz $Grain($idx:oz)
         if {[strlen $oz] == 0} { set oz 0 }
         set name $Grain($idx:name)
         set weight [expr {$lb + 1.000*($oz/16.000)}]
         set percent $Grain($idx:percent)
         set color [lindex $grains($name) 1]
         set srmpartial [partialcolor $color $weight $batchsize]
         set srmpercent [format %0.02f [expr {100 * ($srmpartial/$SRM)}]]

         if {![streqx $lb "" 0]} { puts -nonewline $f "$lb lbs " }
         if {![streqx $oz "" 0]} { puts -nonewline $f "$oz oz " }
         puts $f "$name      ($percent % of wt // $srmpercent % of color)"
      }
      puts $f ""
      global stepmash
      puts $f $stepmash
      puts $f ""


      puts $f "HOP SCHEDULE"
      puts $f ""

      foreach idx $Hop() {
         set goodhop 1
         foreach tag [list amount variety aa time] {
            if {[strlen $Hop($idx:$tag)] == 0} { set goodhop 0 }
         }
         if {$goodhop} {
            puts -nonewline $f "$Hop($idx:amount) oz "
            puts -nonewline $f "$Hop($idx:variety) "
            puts -nonewline $f "$Hop($idx:aa) %AA "
            puts -nonewline $f "at $Hop($idx:time)  "
            puts $f "($Hop($idx:ibu) IBUs // $Hop($idx:percent) % of total IBUs)"
         }
      }
      puts $f ""

      if {![streqx $yeast "" 0]} {
         puts $f "Yeast:  $yeast"
      } else {
         puts $f "Yeast:  not specified"
      }
      puts $f ""

      global display_ibus
      puts $f "TARGET SPECIFICATIONS"
      puts $f ""
      puts $f "     Original Gravity:  $ogtarget ($ogtargpl) (range: $ogrange)"
      puts $f "Final Gravity (range):  $fgmin-$fgmax"
      puts $f "          Color (SRM):  $srmtarget (range:  $srmrange)"
      puts $f "    Bitterness (IBUs):  $display_ibus IBUs (calculated) (range: $iburange)"
      puts $f "           Batch size:  $batchsize gallons"
      puts $f ""

      if {$brewed} {
         puts $f "BREW DAY (AND FINAL) LOG DATA"
         puts $f ""

         puts $f "           Strike liquor temp:  $striketemp F"
         puts $f "         Strike liquor volume:  $strikevol gallons"
         puts $f "  Mash temperature (measured):  $actualmashtemp F"
         puts $f "    Sparge liquor temperature:  $spargetemp F"

         puts $f "First/last wort (P):  $firstwort / $lastwort"
         puts $f "             Post-boil volume:  $finalvol gallons"
         puts $f "              OG // FG // ABV:  $ogactual // $fgactual // $abv %v/v"
         regsub {.* = } $mashppg {} newppg
         puts $f "              Mash efficiency:  $newmasheff% // $newppg pts/lb/gal"
      }

      puts $f ""
   }
   close $f
   tk_messageBox -message "Saved to $fn" -type ok
}


proc make_report_TeX {} {
   global topdir texbin
   global ibutarget iburange
   global Name            ;# Name of brew
   global Date            ;# Date brewed
   global bjcp            ;# 1 or 0 --- is this a BJCP style brew?
   global Style           ;# Style name (BJCP or otherwise)
   global grains          ;# holds the grain database
   global Grain           ;# grain bill data:  Grain(), Grain($idx:$val)
                          ;#    where idx is L1..L5 R1..R5; val is name lb oz
   global Hop             ;# hop sched:  Hop() H1..H5; amount variety aa time
   global yeast           ;# yeast
   global savestyledata   ;# string to save for yeast style data
   global ogtarget ogtargpl ogrange fgmin fgmax srmtarget srmrange
                          ;#
                          ;# BREW DAY LOG DATA
                          ;#
   global striketemp      ;# strike liquor temp (deg. F)
   global strikevol       ;# strike liquor volume (gallons)
   global actualmashtemp  ;# mash temperature (measured, deg. F)
   global spargetemp      ;# sparge liquor temperature
   global firstwort       ;# first wort (info only so far)
   global lastwort        ;# last wort (info only so far)
   global preboilvol      ;# pre-boil volume (gallons)
   global preboilsg       ;# pre-boil gravity
   global pbsgdp
   global ogactual        ;# post-boil specific gravity
   global fgactual        ;# final gravity (post-fermentation)
   global finalvol        ;# post-boil volume
   global abv endmasheff mashppg
                          ;#
                          ;# USER CONFIGURATION DATA
                          ;#
   global batchsize       ;# intended batch size (gallons)
   global boilvol         ;# boil volume (gallons) for hop calculations
   global hopmethod       ;# Rager/Tinseth
   global masheff         ;# estimated mash efficiency (e.g., 75%)
                          ;#
                          ;# DATA FOR CALCULATING STRIKE TEMP/VOLUME FOR
                          ;# A SINGLE-INFUSION MASH
                          ;#
   global sh_grain        ;# specific heat of grain
   global ti_tun          ;# thermal inertia of mash tun
   global boil_temp       ;# boiling point of water ; by default, 212 deg. F
   global ambient_temp    ;# ambient temperature (i.e., GRAIN temperature)
   global mash_thick      ;# desired mash thickness (qts/lb)
   global mashtemp        ;# desired mash temperature
   global brewcomments
   global SRM
   global newmasheff


# From this point on, the code here gets a bit ugly....

   if {[string length $Date]} {
      set date [clock format [clock scan $Date] -format %Y-%h-%d]
      set brewed 1
   } else {
      set date "Brew Pending"
      set brewed 0
   }

   if {$texbin < 0} {
      set opt [tk_messageBox -type yesno -message {The typeset logs use
plain TeX---either "normal" TeX or pdfTeX.  The only difference with
pdfTeX is some minimal use of color.  Are you using pdfTeX or just
plain TeX?  Select Yes for pdfTeX, No for regular (plain) TeX:}]

      if {[string compare -nocase $opt yes] == 0} {
         set pdftex 1
      } else {
         set pdftex 0
      }
   } else {
      set pdftex $texbin
   }
   cd $topdir
   set fn [tk_getSaveFile -initialdir .]
   set f [open $fn w]

   puts $f {\input gtbrewmac}
   if {$pdftex} {
      puts $f {\input pdfcolor}
      puts $f {\line{{\lgboldtext\NavyBlue GTbrew2 Brew Report (\TeX)\Black\hfill}}}
   } else {
      puts $f {\input gtbrewmac}
      puts {\line{{\lgboldtext GTbrew2 Brew Report (\TeX)\hfill}}}
   }
   puts $f {\vskip8pt}
   puts $f ""

   puts $f "\\name{$Name}"
   puts $f "\\datebrewed{$date}"
   if {$bjcp} {
      puts $f "\\bjcpstyle{$Style}"
   } else {
      if {[strlen $Style]} {
      puts $f "\\genericstyle{$Style}" }
   }
   puts $f {\vskip8pt}
   puts $f ""

   if {[check_comments]} {
      puts $f "\\begincomments"

      set cmtlist [split $brewcomments \n]
      foreach s $cmtlist {
      # fix special chars for TeX:  fixing:  # $ % & ... skiping: _ { }

      regsub -all {#} $s {\\#} s2 ; set s $s2
      regsub -all {# } $s {#\\ } s2 ; set s $s2

      regsub -all {\$} $s {\\$} s2 ; set s $s2

      regsub -all {%} $s {\\%} s2 ; set s $s2
      regsub -all {% } $s {%\\ } s2 ; set s $s2

      regsub -all {\&} $s {\\\&} s2 ; set s $s2
      regsub -all {\\\& } $s {\\\&\\ } s2 ; set s $s2

      puts $f $s
      }

      puts $f "\\endcomments"
      puts $f {\vskip8pt}
      puts $f ""
   }

   puts $f {\heading{Grain Bill:}}
   puts $f {\vskip8pt}
   puts $f {}
   puts $f {\halign{#~~~~\hfill&#\hfill&~#~&#\hfill\cr%}}

   set maxlen 0
   foreach idx $Grain() {
      if {![grain_valid $idx]} { continue }
      set len [strlen $Grain($idx:name)]
      if {$len > $maxlen} { set maxlen $len }
   }
   foreach idx $Grain() {
      if {![grain_valid $idx]} { continue }

      set lb $Grain($idx:lb)
      if {[strlen $lb] == 0} { set lb 0 }
      set oz $Grain($idx:oz)
      if {[strlen $oz] == 0} { set oz 0 }
      set name $Grain($idx:name)
      set weight [expr {$lb + 1.000*($oz/16.000)}]
      set percent $Grain($idx:percent)
      set color [lindex $grains($name) 1]
      set srmpartial [partialcolor $color $weight $batchsize]
      set srmpercent [format %0.02f [expr {100 * ($srmpartial/$SRM)}]]

      if {![streqx $lb "" 0]} { puts -nonewline $f "$lb lbs " }
      if {![streqx $oz "" 0]} { puts -nonewline $f "$oz oz " }
      puts $f "$name& ($percent\\%\\ of wt& // &$srmpercent\\%\\ of color)\\cr"
   }
   puts $f "%{
}"
   puts $f {\vskip8pt}
   puts $f ""

   global texstepmash
   puts $f $texstepmash
   puts $f {\vskip8pt}
   puts $f ""


   puts $f {\heading{Hop Schedule:}}
   puts $f {\vskip8pt}
   puts $f ""

   puts $f {\halign{#\hfill&~~~#\hfill&~~~#\hfill\cr%}}
   foreach idx $Hop() {
      set goodhop 1
      foreach tag [list amount variety aa time] {
         if {[strlen $Hop($idx:$tag)] == 0} { set goodhop 0 }
      }
      if {$goodhop} {
         puts -nonewline $f "$Hop($idx:amount) oz "
         set tmpn $Hop($idx:variety)
         set tmpa $Hop($idx:aa)
         regsub " *\\\($tmpa *%\\\)" $tmpn {} hopname
         puts -nonewline $f "$hopname&"
         puts -nonewline $f "$Hop($idx:aa) \\%AA "
         puts -nonewline $f "at $Hop($idx:time)& "
         puts $f "($Hop($idx:ibu) IBUs // $Hop($idx:percent)\\%\\ of total IBUs)\\cr"
      }
   }
    puts $f {%{
}}
   puts $f {\vskip8pt}
   puts $f ""

   if {![streqx $yeast "" 0]} {
      puts $f "{\\boldtext Yeast:}  $yeast"
   } else {
      puts $f "{\\boldtext Yeast:}  not specified"
   }
   puts $f {\vskip8pt}
   puts $f ""

   global display_ibus
   puts $f {\heading{Target Specifications:}}
   puts $f {\vskip8pt}
   puts $f ""


   regsub -- {-} $ogrange {--} ogr
   regsub -- {-} $srmrange {--} srmr
   regsub -- {-} $iburange {--} ibur
   regsub { .*} $ogtargpl {} ogpl

   puts $f {\halign{#\hfill&~~~#\hfill&~~~#\hfill\cr%}}
   puts $f "Original Gravity:&  $ogtarget ($ogpl\\degp) (range: $ogr)\\cr"
   puts $f "Final Gravity (range):&  $fgmin--$fgmax\\cr"
   puts $f "Color (SRM):&  $srmtarget (range:  $srmr)\\cr"
   puts $f "Bitterness (IBUs):&  $display_ibus IBUs (calculated) (range: $ibur)\\cr"
   puts $f "Batch size:&  $batchsize gallons\\cr"
   puts $f {%{
}}
   puts $f {\vskip8pt}
   puts $f ""

   if {$brewed} {
      puts $f "Brew Day (and final) Log Data"
      puts $f {\vskip8pt}
      puts $f ""

      puts $f {\halign{#\hfill&~~~#\hfill&~~~#\hfill\cr%}}
      puts $f "Strike liquor temp:&  $striketemp\\degf\\cr"
      puts $f "Strike liquor volume:&  $strikevol gallons\\cr"
      puts $f "Mash temperature (measured):&  $actualmashtemp\\degf\\cr"
      puts $f "    Sparge liquor temperature:&  $spargetemp\\degf\\cr"

      puts $f "First/last wort:& $firstwort\\degp\\ / $lastwort\\degp\\cr"
      puts $f "Post-boil volume:& $finalvol gallons\\cr"
      regsub { .*} $abv {} newabv
      puts $f "OG // FG // ABV:& $ogactual // $fgactual // $newabv\\% v/v\\cr"
      regsub {.* = } $mashppg {} newppg

      puts $f "Mash efficiency:& $newmasheff\\%\\ // $newppg pts/lb/gal\\cr"
      puts $f {%{
}}
   }



   puts $f {\vfill\eject\bye}
   puts $f ""
   close $f
   tk_messageBox -message "Saved to $fn" -type ok
}


